@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +28,7 @@
 enum {
     COMMIT_TEXT,
     FORWARD_KEY_EVENT,
+    DELETE_SURROUNDING_TEXT,
     UPDATE_PREEDIT_TEXT,
     SHOW_PREEDIT_TEXT,
     HIDE_PREEDIT_TEXT,
@@ -153,6 +155,18 @@ bus_engine_proxy_class_init (BusEngineProxyClass *klass)
             G_TYPE_UINT,
             G_TYPE_UINT);
 
+    engine_signals[DELETE_SURROUNDING_TEXT] =
+        g_signal_new (I_("delete-surrounding-text"),
+            G_TYPE_FROM_CLASS (klass),
+            G_SIGNAL_RUN_LAST,
+            0,
+            NULL, NULL,
+            ibus_marshal_VOID__INT_UINT,
+            G_TYPE_NONE,
+            2,
+            G_TYPE_INT,
+            G_TYPE_UINT);
+
     engine_signals[UPDATE_PREEDIT_TEXT] =
         g_signal_new (I_("update-preedit-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -356,7 +370,10 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
 {
     g_assert (BUS_IS_ENGINE_PROXY (proxy));
     g_assert (message != NULL);
+    g_assert (ibus_message_get_type (message) == DBUS_MESSAGE_TYPE_SIGNAL);
 
+    const gchar *interface;
+    const gchar *name;
     BusEngineProxy *engine;
     IBusError *error;
     gint i;
@@ -375,21 +392,23 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         { "PageDownLookupTable",    PAGE_DOWN_LOOKUP_TABLE },
         { "CursorUpLookupTable",    CURSOR_UP_LOOKUP_TABLE },
         { "CursorDownLookupTable",  CURSOR_DOWN_LOOKUP_TABLE },
-        { NULL, 0},
     };
 
     engine = BUS_ENGINE_PROXY (proxy);
+    interface = ibus_message_get_interface (message);
+    name = ibus_message_get_member (message);
 
-    for (i = 0; ; i++) {
-        if (signals[i].member == NULL)
-            break;
-        if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, signals[i].member)) {
+    if (interface != NULL && g_strcmp0 (interface, IBUS_INTERFACE_ENGINE) != 0)
+        return FALSE;
+
+    for (i = 0; i < G_N_ELEMENTS (signals); i++) {
+        if (g_strcmp0 (name, signals[i].member) == 0) {
             g_signal_emit (engine, engine_signals[signals[i].signal_id], 0);
             goto handled;
         }
     }
 
-    if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "CommitText")) {
+    if (g_strcmp0 (name, "CommitText") == 0) {
         IBusText *text;
         gboolean retval;
 
@@ -402,7 +421,7 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         g_signal_emit (engine, engine_signals[COMMIT_TEXT], 0, text);
         g_object_unref (text);
     }
-    else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "ForwardKeyEvent")) {
+    else if (g_strcmp0 (name, "ForwardKeyEvent") == 0) {
         guint32 keyval;
         guint32 keycode;
         guint32 states;
@@ -419,11 +438,27 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
             goto failed;
         g_signal_emit (engine,
                        engine_signals[FORWARD_KEY_EVENT],
+                       0,
                        keyval,
                        keycode,
                        states);
     }
-    else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "UpdatePreeditText")) {
+    else if (g_strcmp0 (name, "DeleteSurroundingText") == 0) {
+        gint  offset_from_cursor;
+        guint nchars;
+        gboolean retval;
+
+        retval = ibus_message_get_args (message,
+                                        &error,
+                                        G_TYPE_INT,  &offset_from_cursor,
+                                        G_TYPE_UINT, &nchars,
+                                        G_TYPE_INVALID);
+
+        if (!retval)
+            goto failed;
+        g_signal_emit (engine, engine_signals[DELETE_SURROUNDING_TEXT], 0, offset_from_cursor, nchars);
+    }
+    else if (g_strcmp0 (name, "UpdatePreeditText") == 0) {
         IBusText *text;
         gint cursor_pos;
         gboolean visible;
@@ -443,7 +478,7 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
                        text, cursor_pos, visible);
         g_object_unref (text);
     }
-    else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "UpdateAuxiliaryText")) {
+    else if (g_strcmp0 (name, "UpdateAuxiliaryText") == 0) {
         IBusText *text;
         gboolean visible;
         gboolean retval;
@@ -460,7 +495,7 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         g_signal_emit (engine, engine_signals[UPDATE_AUXILIARY_TEXT], 0, text, visible);
         g_object_unref (text);
     }
-    else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "UpdateLookupTable")) {
+    else if (g_strcmp0 (name, "UpdateLookupTable") == 0) {
         IBusLookupTable *table;
         gboolean visible;
         gboolean retval;
@@ -477,7 +512,7 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         g_signal_emit (engine, engine_signals[UPDATE_LOOKUP_TABLE], 0, table, visible);
         g_object_unref (table);
     }
-    else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "RegisterProperties")) {
+    else if (g_strcmp0 (name, "RegisterProperties") == 0) {
         gboolean retval;
 
         if (engine->prop_list) {
@@ -495,7 +530,7 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         }
         g_signal_emit (engine, engine_signals[REGISTER_PROPERTIES], 0, engine->prop_list);
     }
-    else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "UpdateProperty")) {
+    else if (g_strcmp0 (name, "UpdateProperty") == 0) {
         IBusProperty *prop;
         gboolean retval;
 
@@ -510,9 +545,8 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         g_signal_emit (engine, engine_signals[UPDATE_PROPERTY], 0, prop);
         g_object_unref (prop);
     }
-    else {
+    else
         return FALSE;
-    }
 
 handled:
     g_signal_stop_emission_by_name (engine, "ibus-signal");
