@@ -2,7 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2008 Huang Peng <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2009 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -97,12 +98,10 @@ class Setup(object):
         shortcuts = self.__config.get_value(
                         "general/hotkey", "trigger",
                         ibus.CONFIG_GENERAL_SHORTCUT_TRIGGER_DEFAULT)
-
         button = self.__builder.get_object("button_trigger")
         entry = self.__builder.get_object("entry_trigger")
-        text = "; ".join(shortcuts)
-        entry.set_text(text)
-        entry.set_tooltip_text(text)
+        entry.set_text("; ".join(shortcuts))
+        entry.set_tooltip_text("\n".join(shortcuts))
         button.connect("clicked", self.__shortcut_button_clicked_cb,
                     N_("trigger"), "general/hotkey", "trigger", entry)
 
@@ -112,9 +111,8 @@ class Setup(object):
                         ibus.CONFIG_GENERAL_SHORTCUT_NEXT_ENGINE_DEFAULT)
         button = self.__builder.get_object("button_next_engine")
         entry = self.__builder.get_object("entry_next_engine")
-        text = "; ".join(shortcuts)
-        entry.set_text(text)
-        entry.set_tooltip_text(text)
+        entry.set_text("; ".join(shortcuts))
+        entry.set_tooltip_text("\n".join(shortcuts))
         button.connect("clicked", self.__shortcut_button_clicked_cb,
                     N_("next input method"), "general/hotkey", "next_engine", entry)
 
@@ -124,9 +122,8 @@ class Setup(object):
                         ibus.CONFIG_GENERAL_SHORTCUT_PREV_ENGINE_DEFAULT)
         button = self.__builder.get_object("button_prev_engine")
         entry = self.__builder.get_object("entry_prev_engine")
-        text = "; ".join(shortcuts)
-        entry.set_text(text)
-        entry.set_tooltip_text(text)
+        entry.set_text("; ".join(shortcuts))
+        entry.set_tooltip_text("\n".join(shortcuts))
         button.connect("clicked", self.__shortcut_button_clicked_cb,
                     N_("previous input method"), "general/hotkey", "prev_engine", entry)
 
@@ -143,19 +140,22 @@ class Setup(object):
             self.__config.get_value("panel", "show", 1))
         self.__combobox_panel_show.connect("changed", self.__combobox_panel_show_changed_cb)
 
+        # panel position
+        self.__combobox_panel_position = self.__builder.get_object("combobox_panel_position")
+        self.__combobox_panel_position.set_active(
+            self.__config.get_value("panel", "position", 3))
+        self.__combobox_panel_position.connect("changed", self.__combobox_panel_position_changed_cb)
+
         # custom font
         self.__checkbutton_custom_font = self.__builder.get_object("checkbutton_custom_font")
         self.__checkbutton_custom_font.set_active(
             self.__config.get_value("panel", "use_custom_font", False))
         self.__checkbutton_custom_font.connect("toggled", self.__checkbutton_custom_font_toggled_cb)
 
-        self.__label_custom_font = self.__builder.get_object("label_custom_font")
         self.__fontbutton_custom_font = self.__builder.get_object("fontbutton_custom_font")
         if self.__config.get_value("panel", "use_custom_font", False):
-            self.__label_custom_font.set_sensitive(True)
             self.__fontbutton_custom_font.set_sensitive(True)
         else:
-            self.__label_custom_font.set_sensitive(False)
             self.__fontbutton_custom_font.set_sensitive(False)
         font_name = gtk.settings_get_default().get_property("gtk-font-name")
         font_name = unicode(font_name, "utf-8")
@@ -378,13 +378,16 @@ class Setup(object):
             "panel", "show",
             self.__combobox_panel_show.get_active())
 
+    def __combobox_panel_position_changed_cb(self, combobox):
+        self.__config.set_value(
+            "panel", "position",
+            self.__combobox_panel_position.get_active())
+
     def __checkbutton_custom_font_toggled_cb(self, button):
         if self.__checkbutton_custom_font.get_active():
-            self.__label_custom_font.set_sensitive(True)
             self.__fontbutton_custom_font.set_sensitive(True)
             self.__config.set_value("panel", "use_custom_font", True)
         else:
-            self.__label_custom_font.set_sensitive(False)
             self.__fontbutton_custom_font.set_sensitive(False)
             self.__config.set_value("panel", "use_custom_font", False)
 
