@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -287,7 +288,7 @@ ibus_attr_list_init (IBusAttrList *attr_list)
 static void
 ibus_attr_list_destroy (IBusAttrList *attr_list)
 {
-    g_return_if_fail (IBUS_IS_ATTR_LIST (attr_list));
+    g_assert (IBUS_IS_ATTR_LIST (attr_list));
 
     gint i;
     for (i = 0;; i++) {
@@ -415,8 +416,8 @@ void
 ibus_attr_list_append (IBusAttrList  *attr_list,
                        IBusAttribute *attr)
 {
-    g_return_if_fail (IBUS_IS_ATTR_LIST (attr_list));
-    g_return_if_fail (IBUS_IS_ATTRIBUTE (attr));
+    g_assert (IBUS_IS_ATTR_LIST (attr_list));
+    g_assert (IBUS_IS_ATTRIBUTE (attr));
 
     g_object_ref (attr);
     g_array_append_val (attr_list->attributes, attr);
@@ -426,7 +427,7 @@ IBusAttribute *
 ibus_attr_list_get (IBusAttrList *attr_list,
                     guint         index)
 {
-    g_return_val_if_fail (attr_list != NULL, NULL);
+    g_assert (IBUS_IS_ATTR_LIST (attr_list));
     IBusAttribute *attr = NULL;
 
     if (index < attr_list->attributes->len) {
