@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +27,8 @@
 #include "ibusinternal.h"
 #include "ibusshare.h"
 #include "ibusconnection.h"
+#include "ibusenginedesc.h"
+#include "ibusserializable.h"
 #include "ibusconfig.h"
 
 #define IBUS_BUS_GET_PRIVATE(o)  \
@@ -34,7 +37,6 @@
 enum {
     CONNECTED,
     DISCONNECTED,
-    NAME_OWNER_CHANGED,
     LAST_SIGNAL,
 };
 
@@ -140,21 +142,6 @@ ibus_bus_class_init (IBusBusClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE,
             0);
-#if 0
-    bus_signals[NAME_OWNER_CHANGED] =
-        g_signal_new (I_("name-owner-changed"),
-            G_TYPE_FROM_CLASS (klass),
-            G_SIGNAL_RUN_LAST,
-            0,
-            NULL, NULL,
-            ibus_marshal_VOID__STRING_STRING_STRING,
-            G_TYPE_NONE,
-            3,
-            G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-            G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-            G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE
-            );
-#endif
 }
 
 static void
@@ -224,12 +211,15 @@ _changed_cb (GFileMonitor       *monitor,
              GFileMonitorEvent   event_type,
              IBusBus            *bus)
 {
-    // g_debug ("changed %x", event_type);
+    if (event_type != G_FILE_MONITOR_EVENT_CHANGED &&
+        event_type != G_FILE_MONITOR_EVENT_CREATED &&
+        event_type != G_FILE_MONITOR_EVENT_DELETED)
+        return;
+
     if (ibus_bus_is_connected (bus))
         return;
-    if (event_type == G_FILE_MONITOR_EVENT_CHANGED) {
-        ibus_bus_connect (bus);
-    }
+    
+    ibus_bus_connect (bus);
 }
 
 static void
@@ -379,61 +369,6 @@ ibus_bus_create_input_context (IBusBus      *bus,
     return context;
 }
 
-static void
-ibus_bus_watch_dbus_signal (IBusBus *bus)
-{
-    g_assert (IBUS_IS_BUS (bus));
-
-    const gchar *rule;
-
-    rule = "type='signal'," \
-           "path='" DBUS_PATH_DBUS "'," \
-           "interface='" DBUS_INTERFACE_DBUS "'";
-
-    ibus_bus_add_match (bus, rule);
-
-}
-
-static void
-ibus_bus_unwatch_dbus_signal (IBusBus *bus)
-{
-    g_assert (IBUS_IS_BUS (bus));
-    g_assert (ibus_bus_is_connected (bus));
-
-    const gchar *rule;
-
-    rule = "type='signal'," \
-           "path='" DBUS_PATH_DBUS "'," \
-           "interface='" DBUS_INTERFACE_DBUS "'";
-
-    ibus_bus_remove_match (bus, rule);
-}
-
-void
-ibus_bus_set_watch_dbus_signal (IBusBus        *bus,
-                                gboolean        watch)
-{
-    g_assert (IBUS_IS_BUS (bus));
-
-    IBusBusPrivate *priv;
-    priv = IBUS_BUS_GET_PRIVATE (bus);
-
-    if (priv->watch_dbus_signal == watch)
-        return;
-
-    priv->watch_dbus_signal = watch;
-
-    if (ibus_bus_is_connected (bus)) {
-        if (watch) {
-            ibus_bus_watch_dbus_signal (bus);
-        }
-        else {
-            ibus_bus_unwatch_dbus_signal (bus);
-        }
-    }
-}
-
-
 static gboolean
 ibus_bus_call (IBusBus      *bus,
                const gchar  *name,
@@ -444,7 +379,6 @@ ibus_bus_call (IBusBus      *bus,
                ...)
 {
     g_assert (IBUS_IS_BUS (bus));
-    g_assert (ibus_bus_is_connected (bus));
     g_assert (name != NULL);
     g_assert (path != NULL);
     g_assert (interface != NULL);
@@ -457,6 +391,8 @@ ibus_bus_call (IBusBus      *bus,
     gboolean retval;
     IBusBusPrivate *priv;
 
+    g_return_val_if_fail (ibus_bus_is_connected (bus), FALSE);
+
     priv = IBUS_BUS_GET_PRIVATE (bus);
 
     message = ibus_message_new_method_call (name, path, interface, member);
@@ -514,6 +450,82 @@ ibus_bus_call (IBusBus      *bus,
     return TRUE;
 }
 
+const gchar *
+ibus_bus_current_input_context(IBusBus      *bus)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    gchar *name = NULL;
+    gboolean result;
+
+    result = ibus_bus_call (bus,
+                            DBUS_SERVICE_DBUS,
+                            DBUS_PATH_DBUS,
+                            DBUS_INTERFACE_DBUS,
+                            "CurrentInputContext",
+                            G_TYPE_INVALID,
+                            G_TYPE_STRING, &name,
+                            G_TYPE_INVALID);
+
+    if (result)
+        return name;
+
+    return NULL;
+}
+
+static void
+ibus_bus_watch_dbus_signal (IBusBus *bus)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    const gchar *rule;
+
+    rule = "type='signal'," \
+           "path='" DBUS_PATH_DBUS "'," \
+           "interface='" DBUS_INTERFACE_DBUS "'";
+
+    ibus_bus_add_match (bus, rule);
+
+}
+
+static void
+ibus_bus_unwatch_dbus_signal (IBusBus *bus)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    const gchar *rule;
+
+    rule = "type='signal'," \
+           "path='" DBUS_PATH_DBUS "'," \
+           "interface='" DBUS_INTERFACE_DBUS "'";
+
+    ibus_bus_remove_match (bus, rule);
+}
+
+void
+ibus_bus_set_watch_dbus_signal (IBusBus        *bus,
+                                gboolean        watch)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    IBusBusPrivate *priv;
+    priv = IBUS_BUS_GET_PRIVATE (bus);
+
+    if (priv->watch_dbus_signal == watch)
+        return;
+
+    priv->watch_dbus_signal = watch;
+
+    if (ibus_bus_is_connected (bus)) {
+        if (watch) {
+            ibus_bus_watch_dbus_signal (bus);
+        }
+        else {
+            ibus_bus_unwatch_dbus_signal (bus);
+        }
+    }
+}
+
 const gchar *
 ibus_bus_hello (IBusBus *bus)
 {
@@ -593,8 +605,6 @@ gboolean
 ibus_bus_name_has_owner (IBusBus        *bus,
                          const gchar    *name)
 {
-    g_assert (IBUS_IS_BUS (bus));
-
     gboolean retval;
     gboolean result;
 
@@ -774,16 +784,87 @@ ibus_bus_register_component (IBusBus       *bus,
 #endif
 }
 
+static GList *
+ibus_bus_do_list_engines (IBusBus *bus, gboolean active_engines_only)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    IBusMessage *message, *reply;
+    IBusError *error;
+    gboolean retval;
+    IBusBusPrivate *priv;
+    IBusMessageIter iter, subiter;
+    GList *engines;
+    const gchar* member = active_engines_only ? "ListActiveEngines" : "ListEngines";
+
+    priv = IBUS_BUS_GET_PRIVATE (bus);
+    message = ibus_message_new_method_call (IBUS_SERVICE_IBUS,
+                                            IBUS_PATH_IBUS,
+                                            IBUS_INTERFACE_IBUS,
+                                            member);
+    reply = ibus_connection_send_with_reply_and_block (priv->connection,
+                                                       message,
+                                                       -1,
+                                                       &error);
+    ibus_message_unref (message);
+
+    if (reply == NULL) {
+        g_warning ("%s : %s", error->name, error->message);
+        ibus_error_free (error);
+        return NULL;
+    }
+
+    if ((error = ibus_error_new_from_message (reply)) != NULL) {
+        g_warning ("%s : %s", error->name, error->message);
+        ibus_error_free (error);
+        ibus_message_unref (reply);
+        return NULL;
+    }
+
+    retval = ibus_message_iter_init (reply, &iter);
+    if (!retval) {
+        error = ibus_error_new_from_printf (DBUS_ERROR_INVALID_ARGS,
+                                            "Message does not have arguments!");
+        g_warning ("%s : %s", error->name, error->message);
+        ibus_error_free (error);
+        ibus_message_unref (reply);
+        return NULL;
+    }
+
+    if (!ibus_message_iter_recurse (&iter, IBUS_TYPE_ARRAY, &subiter)) {
+        ibus_message_unref (reply);
+        return NULL;
+    }
+
+    if (dbus_message_iter_get_array_len (&subiter) <= 0) {
+        ibus_message_unref (reply);
+        return NULL;
+    }
+
+    engines = NULL;
+    do {
+        IBusSerializable *object = NULL;
+        if (!ibus_message_iter_get (&subiter, IBUS_TYPE_ENGINE_DESC, &object) || !object) {
+            g_warning ("Unexpected type is returned from %s", member);
+            continue;
+        }
+        engines = g_list_append (engines, object);
+    } while (ibus_message_iter_next (&subiter));
+
+    ibus_message_unref (reply);
+    return engines;
+}
+
 GList *
 ibus_bus_list_engines (IBusBus *bus)
 {
-    return NULL;
+    return ibus_bus_do_list_engines (bus, FALSE);
 }
 
 GList *
 ibus_bus_list_active_engines (IBusBus *bus)
 {
-    return NULL;
+    return ibus_bus_do_list_engines (bus, TRUE);
 }
 
 static void
