@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,7 +28,7 @@ _from_dbus_value (IBusMessageIter   *iter,
     g_assert (iter != NULL);
     g_assert (value != NULL);
 
-    gint type;
+    GType type;
     IBusMessageIter sub_iter;
 
     type = ibus_message_iter_get_arg_type (iter);
@@ -82,7 +83,7 @@ _from_dbus_value (IBusMessageIter   *iter,
         if (type == IBUS_TYPE_ARRAY) {
             GValue v = { 0 };
             IBusMessageIter sub_iter;
-            gint sub_type;
+            GType sub_type;
             GValueArray *array;
 
 
