@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -247,23 +248,25 @@ ibus_connection_destroy (IBusConnection *connection)
                     connection);
     }
 
-    if (!priv->shared && priv->connection) {
-        dbus_connection_close (priv->connection);
-        dbus_connection_unref (priv->connection);
-        priv->connection = NULL;
-        goto _out;
-    }
-
-    if (priv->shared && priv->connection) {
-        g_warn_if_fail (_connections != NULL);
-        if (_connections != NULL) {
-            g_hash_table_remove (_connections, priv->connection);
+    do {
+        if (!priv->shared && priv->connection) {
+            dbus_connection_close (priv->connection);
+            dbus_connection_unref (priv->connection);
+            priv->connection = NULL;
+            break;
         }
-        dbus_connection_unref (priv->connection);
-        priv->connection = NULL;
-        goto _out;
-    }
-_out:
+
+        if (priv->shared && priv->connection) {
+            g_warn_if_fail (_connections != NULL);
+            if (_connections != NULL) {
+                g_hash_table_remove (_connections, priv->connection);
+            }
+            dbus_connection_unref (priv->connection);
+            priv->connection = NULL;
+            break;
+        }
+    } while (0);
+
     parent_class->destroy (IBUS_OBJECT (connection));
 }
 
@@ -755,12 +758,11 @@ ibus_connection_call (IBusConnection     *connection,
     g_assert (path != NULL);
     g_assert (interface != NULL);
     g_assert (member != NULL);
+    g_return_val_if_fail (ibus_connection_is_connected (connection), FALSE);
 
     IBusConnectionPrivate *priv;
     priv = IBUS_CONNECTION_GET_PRIVATE (connection);
 
-    g_assert (dbus_connection_get_is_connected (priv->connection));
-
     IBusMessage *message, *reply;
     IBusError *tmp_error;
     va_list args;
@@ -824,7 +826,7 @@ void
 ibus_connection_flush (IBusConnection *connection)
 {
     g_assert (IBUS_IS_CONNECTION (connection));
-    g_assert (ibus_connection_is_connected (connection));
+    g_return_if_fail (ibus_connection_is_connected (connection));
 
     IBusConnectionPrivate *priv;
 
