@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +33,7 @@ enum {
     DISABLED,
     COMMIT_TEXT,
     FORWARD_KEY_EVENT,
+    DELETE_SURROUNDING_TEXT,
     UPDATE_PREEDIT_TEXT,
     SHOW_PREEDIT_TEXT,
     HIDE_PREEDIT_TEXT,
@@ -53,7 +55,7 @@ enum {
 
 /* BusInputContextPriv */
 struct _IBusInputContextPrivate {
-    void *pad;
+    gboolean own;
 };
 typedef struct _IBusInputContextPrivate IBusInputContextPrivate;
 
@@ -112,6 +114,19 @@ ibus_input_context_new (const gchar     *path,
     return IBUS_INPUT_CONTEXT (obj);
 }
 
+IBusInputContext *
+ibus_input_context_get_input_context (const gchar        *path,
+                                      IBusConnection     *connection)
+{
+    IBusInputContext *context = ibus_input_context_new (path, connection);
+    IBusInputContextPrivate *priv;
+    if (!context)
+        return NULL;
+    priv = IBUS_INPUT_CONTEXT_GET_PRIVATE (context);
+    priv->own = FALSE;
+    return context;
+}
+
 static void
 ibus_input_context_class_init (IBusInputContextClass *klass)
 {
@@ -198,6 +213,27 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             G_TYPE_UINT,
             G_TYPE_UINT);
 
+    /**
+     * IBusInputContext::delete-surrounding-text:
+     * @context: An IBusInputContext.
+     * @offset: the character offset from the cursor position of the text to be deleted.
+     *   A negative value indicates a position before the cursor.
+     * @n_chars: the number of characters to be deleted.
+     *
+     * Emitted to delete surrounding text event from IME to client of IME.
+     */
+    context_signals[DELETE_SURROUNDING_TEXT] =
+        g_signal_new (I_("delete-surrounding-text"),
+            G_TYPE_FROM_CLASS (klass),
+            G_SIGNAL_RUN_LAST,
+            0,
+            NULL, NULL,
+            ibus_marshal_VOID__INT_UINT,
+            G_TYPE_NONE,
+            2,
+            G_TYPE_INT,
+            G_TYPE_UINT);
+
     /**
      * IBusInputContext::update-preedit-text:
      * @context: An IBusInputContext.
@@ -449,12 +485,16 @@ ibus_input_context_init (IBusInputContext *context)
 {
     IBusInputContextPrivate *priv;
     priv = IBUS_INPUT_CONTEXT_GET_PRIVATE (context);
+    priv->own = TRUE;
 }
 
 static void
 ibus_input_context_real_destroy (IBusInputContext *context)
 {
-    if (ibus_proxy_get_connection ((IBusProxy *) context) != NULL) {
+    IBusInputContextPrivate *priv;
+    priv = IBUS_INPUT_CONTEXT_GET_PRIVATE (context);
+
+    if (priv->own && ibus_proxy_get_connection ((IBusProxy *) context) != NULL) {
         ibus_proxy_call (IBUS_PROXY (context),
                          "Destroy",
                          G_TYPE_INVALID);
@@ -469,10 +509,13 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (proxy));
     g_assert (message != NULL);
+    g_assert (ibus_message_get_type (message) == DBUS_MESSAGE_TYPE_SIGNAL);
 
     IBusInputContext *context;
     IBusError *error = NULL;
     gint i;
+    const gchar *interface;
+    const gchar *name;
 
     context = IBUS_INPUT_CONTEXT (proxy);
 
@@ -480,193 +523,205 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
         const gchar *member;
         guint signal_id;
     } signals [] = {
-        { "Enabled",                ENABLED                 },
-        { "Disabled",               DISABLED                },
-        { "ShowPreeditText",        SHOW_PREEDIT_TEXT       },
-        { "HidePreeditText",        HIDE_PREEDIT_TEXT       },
-        { "ShowAuxiliaryText",      SHOW_AUXILIARY_TEXT     },
-        { "HideAuxiliaryText",      HIDE_AUXILIARY_TEXT     },
-        { "ShowLookupTable",        SHOW_LOOKUP_TABLE       },
-        { "HideLookupTable",        HIDE_LOOKUP_TABLE       },
-        { "PageUpLookupTable",      PAGE_UP_LOOKUP_TABLE    },
-        { "PageDownLookupTable",    PAGE_DOWN_LOOKUP_TABLE  },
-        { "CursorUpLookupTable",    CURSOR_UP_LOOKUP_TABLE  },
-        { "CursorDownLookupTable",  CURSOR_DOWN_LOOKUP_TABLE},
-        { NULL, 0},
+        { "Enabled",                ENABLED                  },
+        { "Disabled",               DISABLED                 },
+        { "ShowPreeditText",        SHOW_PREEDIT_TEXT        },
+        { "HidePreeditText",        HIDE_PREEDIT_TEXT        },
+        { "ShowAuxiliaryText",      SHOW_AUXILIARY_TEXT      },
+        { "HideAuxiliaryText",      HIDE_AUXILIARY_TEXT      },
+        { "ShowLookupTable",        SHOW_LOOKUP_TABLE        },
+        { "HideLookupTable",        HIDE_LOOKUP_TABLE        },
+        { "PageUpLookupTable",      PAGE_UP_LOOKUP_TABLE     },
+        { "PageDownLookupTable",    PAGE_DOWN_LOOKUP_TABLE   },
+        { "CursorUpLookupTable",    CURSOR_UP_LOOKUP_TABLE   },
+        { "CursorDownLookupTable",  CURSOR_DOWN_LOOKUP_TABLE },
     };
 
-    for (i = 0; ; i++) {
-        if (signals[i].member == NULL)
+    do {
+        interface = ibus_message_get_interface (message);
+        name = ibus_message_get_member (message);
+
+        if (interface != NULL && g_strcmp0 (interface, IBUS_INTERFACE_INPUT_CONTEXT) != 0) {
+            error = ibus_error_new_from_printf (DBUS_ERROR_FAILED,
+                                                "Signal %s.%s is not handled", interface, name);
             break;
-        if (ibus_message_is_signal (message,
-                                    IBUS_INTERFACE_INPUT_CONTEXT,
-                                    signals[i].member)) {
-            g_signal_emit (context, context_signals[signals[i].signal_id], 0);
-            goto handled;
         }
+
+        if (g_strcmp0 (name, "CommitText") == 0) {
+            IBusText *text;
+            gboolean retval;
+
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            IBUS_TYPE_TEXT, &text,
+                                            G_TYPE_INVALID);
+            if (retval) {
+                g_signal_emit (context, context_signals[COMMIT_TEXT], 0, text);
+                g_object_unref (text);
+            }
+            break;
+        }
+        if (g_strcmp0 (name, "UpdatePreeditText") == 0) {
+            IBusText *text;
+            gint32 cursor_pos;
+            gboolean visible;
+            gboolean retval;
+
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            IBUS_TYPE_TEXT, &text,
+                                            G_TYPE_UINT, &cursor_pos,
+                                            G_TYPE_BOOLEAN, &visible,
+                                            G_TYPE_INVALID);
+
+            if (retval) {
+                g_signal_emit (context,
+                               context_signals[UPDATE_PREEDIT_TEXT],
+                               0,
+                               text,
+                               cursor_pos,
+                               visible);
+                g_object_unref (text);
+            }
+            break;
+        }
+
+        for (i = 0; i < G_N_ELEMENTS (signals); i++) {
+            if (g_strcmp0 (name, signals[i].member) == 0) {
+                g_signal_emit (context, context_signals[signals[i].signal_id], 0);
+                break;
+            }
+        }
+
+        if (i < G_N_ELEMENTS (signals))
+            break;
+        if (g_strcmp0 (name, "UpdateAuxiliaryText") == 0) {
+            IBusText *text;
+            gboolean visible;
+            gboolean retval;
+
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            IBUS_TYPE_TEXT, &text,
+                                            G_TYPE_BOOLEAN, &visible,
+                                            G_TYPE_INVALID);
+
+            if (retval) {
+                g_signal_emit (context,
+                               context_signals[UPDATE_AUXILIARY_TEXT],
+                               0,
+                               text,
+                               visible);
+                g_object_unref (text);
+            }
+        }
+        else if (g_strcmp0 (name, "UpdateLookupTable") == 0) {
+            IBusLookupTable *table;
+            gboolean visible;
+            gboolean retval;
+
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            IBUS_TYPE_LOOKUP_TABLE, &table,
+                                            G_TYPE_BOOLEAN, &visible,
+                                            G_TYPE_INVALID);
+
+            if (retval) {
+                g_signal_emit (context,
+                               context_signals[UPDATE_LOOKUP_TABLE],
+                               0,
+                               table,
+                               visible);
+                g_object_unref (table);
+            }
+        }
+        else if (g_strcmp0 (name, "RegisterProperties") == 0) {
+            IBusPropList *prop_list;
+            gboolean retval;
+
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            IBUS_TYPE_PROP_LIST, &prop_list,
+                                            G_TYPE_INVALID);
+
+            if (retval) {
+                g_signal_emit (context,
+                               context_signals[REGISTER_PROPERTIES],
+                               0,
+                               prop_list);
+                g_object_unref (prop_list);
+            }
+        }
+        else if (g_strcmp0 (name, "UpdateProperty") == 0) {
+            IBusProperty *prop;
+            gboolean retval;
+
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            IBUS_TYPE_PROPERTY, &prop,
+                                            G_TYPE_INVALID);
+            if (retval) {
+                g_signal_emit (context, context_signals[UPDATE_PROPERTY], 0, prop);
+                g_object_unref (prop);
+            }
+        }
+        else if (g_strcmp0 (name, "ForwardKeyEvent") == 0) {
+            guint32 keyval;
+            guint32 keycode;
+            guint32 state;
+            gboolean retval;
+
+
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            G_TYPE_UINT, &keyval,
+                                            G_TYPE_UINT, &keycode,
+                                            G_TYPE_UINT, &state,
+                                            G_TYPE_INVALID);
+
+            if (retval) {
+                g_signal_emit (context,
+                               context_signals[FORWARD_KEY_EVENT],
+                               0,
+                               keyval,
+                               keycode,
+                               state | IBUS_FORWARD_MASK);
+            }
+        }
+        else if (g_strcmp0 (name, "DeleteSurroundingText") == 0) {
+            gint offset_from_cursor;
+            guint nchars;
+            gboolean retval;
+            retval = ibus_message_get_args (message,
+                                            &error,
+                                            G_TYPE_INT, &offset_from_cursor,
+                                            G_TYPE_UINT, &nchars,
+                                            G_TYPE_INVALID);
+
+
+            if (retval) {
+                g_signal_emit (context,
+                               context_signals[DELETE_SURROUNDING_TEXT],
+                               0,
+                               offset_from_cursor,
+                               nchars);
+            }
+        }
+        else {
+            error = ibus_error_new_from_printf (DBUS_ERROR_FAILED,
+                                                "Signal %s.%s is not handled", interface, name);
+            break;
+        }
+    } while (0);
+
+    if (error == NULL) {
+        g_signal_stop_emission_by_name (context, "ibus-signal");
+        return TRUE;
     }
 
-    if (ibus_message_is_signal (message,
-                                IBUS_INTERFACE_INPUT_CONTEXT,
-                                "CommitText")) {
-        IBusText *text;
-        gboolean retval;
-
-        retval = ibus_message_get_args (message,
-                                        &error,
-                                        IBUS_TYPE_TEXT, &text,
-                                        G_TYPE_INVALID);
-        if (!retval)
-            goto failed;
-
-        g_signal_emit (context, context_signals[COMMIT_TEXT], 0, text);
-        g_object_unref (text);
-    }
-    else if (ibus_message_is_signal (message,
-                                     IBUS_INTERFACE_INPUT_CONTEXT,
-                                     "UpdatePreeditText")) {
-        IBusText *text;
-        gint32 cursor_pos;
-        gboolean visible;
-        gboolean retval;
-
-        retval = ibus_message_get_args (message,
-                                        &error,
-                                        IBUS_TYPE_TEXT, &text,
-                                        G_TYPE_UINT, &cursor_pos,
-                                        G_TYPE_BOOLEAN, &visible,
-                                        G_TYPE_INVALID);
-
-        if (!retval)
-            goto failed;
-
-        g_signal_emit (context,
-                       context_signals[UPDATE_PREEDIT_TEXT],
-                       0,
-                       text,
-                       cursor_pos,
-                       visible);
-        g_object_unref (text);
-    }
-    else if (ibus_message_is_signal (message,
-                                     IBUS_INTERFACE_INPUT_CONTEXT,
-                                     "ForwardKeyEvent")) {
-        guint32 keyval;
-        guint32 keycode;
-        guint32 state;
-        gboolean retval;
-
-        retval = ibus_message_get_args (message,
-                                        &error,
-                                        G_TYPE_UINT, &keyval,
-                                        G_TYPE_UINT, &keycode,
-                                        G_TYPE_UINT, &state,
-                                        G_TYPE_INVALID);
-
-        if (!retval)
-            goto failed;
-        g_signal_emit (context,
-                       context_signals[FORWARD_KEY_EVENT],
-                       0,
-                       keyval,
-                       keycode,
-                       state | IBUS_FORWARD_MASK);
-    }
-    else if (ibus_message_is_signal (message,
-                                     IBUS_INTERFACE_INPUT_CONTEXT,
-                                     "UpdateAuxiliaryText")) {
-        IBusText *text;
-        gboolean visible;
-        gboolean retval;
-
-        retval = ibus_message_get_args (message,
-                                        &error,
-                                        IBUS_TYPE_TEXT, &text,
-                                        G_TYPE_BOOLEAN, &visible,
-                                        G_TYPE_INVALID);
-
-        if (!retval)
-            goto failed;
-
-        g_signal_emit (context,
-                       context_signals[UPDATE_AUXILIARY_TEXT],
-                       0,
-                       text,
-                       visible);
-        g_object_unref (text);
-    }
-    else if (ibus_message_is_signal (message,
-                                     IBUS_INTERFACE_INPUT_CONTEXT,
-                                     "UpdateLookupTable")) {
-        IBusLookupTable *table;
-        gboolean visible;
-        gboolean retval;
-
-        retval = ibus_message_get_args (message,
-                                        &error,
-                                        IBUS_TYPE_LOOKUP_TABLE, &table,
-                                        G_TYPE_BOOLEAN, &visible,
-                                        G_TYPE_INVALID);
-
-        if (!retval)
-            goto failed;
-
-        g_signal_emit (context,
-                       context_signals[UPDATE_LOOKUP_TABLE],
-                       0,
-                       table,
-                       visible);
-        g_object_unref (table);
-    }
-    else if (ibus_message_is_signal (message,
-                                     IBUS_INTERFACE_INPUT_CONTEXT,
-                                     "RegisterProperties")) {
-        IBusPropList *prop_list;
-        gboolean retval;
-
-        retval = ibus_message_get_args (message,
-                                        &error,
-                                        IBUS_TYPE_PROP_LIST, &prop_list,
-                                        G_TYPE_INVALID);
-
-        if (!retval)
-            goto failed;
-
-        g_signal_emit (context,
-                       context_signals[REGISTER_PROPERTIES],
-                       0,
-                       prop_list);
-        g_object_unref (prop_list);
-    }
-    else if (ibus_message_is_signal (message,
-                                     IBUS_INTERFACE_INPUT_CONTEXT,
-                                     "UpdateProperty")) {
-        IBusProperty *prop;
-        gboolean retval;
-
-        retval = ibus_message_get_args (message,
-                                        &error,
-                                        IBUS_TYPE_PROPERTY, &prop,
-                                        G_TYPE_INVALID);
-        if (!retval)
-            goto failed;
-
-        g_signal_emit (context, context_signals[UPDATE_PROPERTY], 0, prop);
-        g_object_unref (prop);
-    }
-    else {
-        return FALSE;
-    }
-
-handled:
-    g_signal_stop_emission_by_name (context, "ibus-signal");
-    return TRUE;
-
-failed:
-    if (error) {
-        g_warning ("%s: %s", error->name, error->message);
-        ibus_error_free (error);
-    }
+    /* some error happens */
+    g_warning ("%s: %s", error->name, error->message);
+    ibus_error_free (error);
     return FALSE;
 }
 
@@ -806,6 +861,84 @@ ibus_input_context_property_hide (IBusInputContext *context,
                      G_TYPE_INVALID);
 }
 
+gboolean
+ibus_input_context_is_enabled (IBusInputContext *context)
+{
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    gboolean retval = FALSE;
+
+    IBusMessage *reply_message;
+    IBusError *error = NULL;
+
+    reply_message = ibus_proxy_call_with_reply_and_block ((IBusProxy *) context,
+                                                          "IsEnabled",
+                                                          -1,
+                                                          &error,
+                                                          G_TYPE_INVALID);
+    if (!reply_message) {
+        g_debug ("%s: %s", error->name, error->message);
+        ibus_error_free (error);
+        return FALSE;
+    }
+
+    if (!ibus_message_get_args (reply_message,
+                                &error,
+                                G_TYPE_BOOLEAN, &retval,
+                                G_TYPE_INVALID)) {
+        g_debug ("%s: %s", error->name, error->message);
+        ibus_error_free (error);
+        retval = FALSE;
+    }
+    ibus_message_unref (reply_message);
+
+    return retval;
+}
+
+IBusEngineDesc *
+ibus_input_context_get_engine (IBusInputContext *context)
+{
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    IBusMessage *reply_message;
+    IBusError *error = NULL;
+    IBusSerializable *object = NULL;
+
+    reply_message = ibus_proxy_call_with_reply_and_block ((IBusProxy *) context,
+                                                          "GetEngine",
+                                                          -1,
+                                                          &error,
+                                                          G_TYPE_INVALID);
+    if (!reply_message) {
+        g_debug ("%s: %s", error->name, error->message);
+        ibus_error_free (error);
+        return NULL;
+    }
+
+    if (!ibus_message_get_args (reply_message,
+                                &error,
+                                IBUS_TYPE_ENGINE_DESC, &object,
+                                G_TYPE_INVALID)) {
+        g_debug ("%s: %s", error->name, error->message);
+        ibus_error_free (error);
+        ibus_message_unref (reply_message);
+        return NULL;
+    }
+    ibus_message_unref (reply_message);
+
+    return IBUS_ENGINE_DESC (object);
+}
+
+void
+ibus_input_context_set_engine (IBusInputContext *context,
+                               const gchar *name)
+{
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+
+    ibus_proxy_call ((IBusProxy *) context,
+                     "SetEngine",
+                     G_TYPE_STRING, &name,
+                     G_TYPE_INVALID);
+}
+
 #define DEFINE_FUNC(name,Name)                              \
     void                                                    \
     ibus_input_context_##name (IBusInputContext *context)   \
