@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,6 +34,7 @@
 #define __IBUS_INPUT_CONTEXT_H_
 
 #include "ibusproxy.h"
+#include "ibusenginedesc.h"
 
 /*
  * Type macros.
@@ -89,6 +91,20 @@ GType        ibus_input_context_get_type    (void);
 IBusInputContext
             *ibus_input_context_new         (const gchar        *path,
                                              IBusConnection     *connection);
+
+/**
+ * ibus_input_context_get_input_context:
+ * @path: The path to the object that emitting the signal.
+ * @connection: An IBusConnection.
+ * @returns: An existing IBusInputContext.
+ *
+ * Gets an existing IBusInputContext.
+ */
+IBusInputContext
+            *ibus_input_context_get_input_context
+                                            (const gchar        *path,
+                                             IBusConnection     *connection);
+
 /**
  * ibus_input_context_process_key_event:
  * @context: An IBusInputContext.
@@ -205,6 +221,37 @@ void         ibus_input_context_enable      (IBusInputContext   *context);
  */
 void         ibus_input_context_disable     (IBusInputContext   *context);
 
+
+/**
+ * ibus_input_context_is_ensabled:
+ * @context: An IBusInputContext.
+ * @returns: TRUE if the IME is enabled on the context. 
+ *
+ * Returns TRUE if the IME is enabled on the context.
+ */
+gboolean     ibus_input_context_is_enabled  (IBusInputContext   *context);
+
+/**
+ * ibus_input_context_get_engine:
+ * @context: An IBusInputContext.
+ * @returns: An IME engine description for the context
+ *
+ * Returns an IME engine description for the context.
+ */
+IBusEngineDesc
+            *ibus_input_context_get_engine  (IBusInputContext   *context);
+
+/**
+ * ibus_input_context_set_engine:
+ * @context: An IBusInputContext.
+ * @name: A name of the engine.
+ *
+ * Invoked when the IME engine is changed.
+ */
+void         ibus_input_context_set_engine  (IBusInputContext   *context,
+                                             const gchar *name);
+
+
 G_END_DECLS
 #endif
 
