@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -73,7 +74,7 @@ ibus_lookup_table_class_init (IBusLookupTableClass *klass)
     serializable_class->deserialize = (IBusSerializableDeserializeFunc) ibus_lookup_table_deserialize;
     serializable_class->copy        = (IBusSerializableCopyFunc) ibus_lookup_table_copy;
 
-    g_string_append (serializable_class->signature, "uubbavav");
+    g_string_append (serializable_class->signature, "uubbiavav");
 }
 
 static void
@@ -136,6 +137,9 @@ ibus_lookup_table_serialize (IBusLookupTable *table,
     retval = ibus_message_iter_append (iter, G_TYPE_BOOLEAN, &table->round);
     g_return_val_if_fail (retval, FALSE);
 
+    retval = ibus_message_iter_append (iter, G_TYPE_INT, &table->orientation);
+    g_return_val_if_fail (retval, FALSE);
+
     // append candidates
     retval = ibus_message_iter_open_container (iter,
                                                IBUS_TYPE_ARRAY,
@@ -209,6 +213,10 @@ ibus_lookup_table_deserialize (IBusLookupTable *table,
     g_return_val_if_fail (retval, FALSE);
     ibus_message_iter_next (iter);
 
+    retval = ibus_message_iter_get (iter, G_TYPE_INT, &table->orientation);
+    g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
+
     // deserialize candidates
     retval = ibus_message_iter_recurse (iter, IBUS_TYPE_ARRAY, &array_iter);
     g_return_val_if_fail (retval, FALSE);
@@ -305,16 +313,25 @@ ibus_lookup_table_new (guint page_size,
     table->cursor_pos = cursor_pos;
     table->cursor_visible = cursor_visible;
     table->round = round;
+    table->orientation = IBUS_ORIENTATION_SYSTEM;
 
     return table;
 }
 
+guint
+ibus_lookup_table_get_number_of_candidates (IBusLookupTable *table)
+{
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
+
+    return table->candidates->len;
+}
+
 void
 ibus_lookup_table_append_candidate (IBusLookupTable *table,
                                     IBusText        *text)
 {
-    g_return_if_fail (IBUS_IS_LOOKUP_TABLE (table));
-    g_return_if_fail (IBUS_IS_TEXT (text));
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
+    g_assert (IBUS_IS_TEXT (text));
 
     g_object_ref (text);
     g_array_append_val (table->candidates, text);
@@ -324,7 +341,7 @@ IBusText *
 ibus_lookup_table_get_candidate (IBusLookupTable *table,
                                  guint            index)
 {
-    g_return_val_if_fail (IBUS_IS_LOOKUP_TABLE (table), NULL);
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
 
     if (index >= table->candidates->len)
         return NULL;
@@ -336,8 +353,8 @@ void
 ibus_lookup_table_append_label (IBusLookupTable *table,
                                 IBusText        *text)
 {
-    g_return_if_fail (IBUS_IS_LOOKUP_TABLE (table));
-    g_return_if_fail (IBUS_IS_TEXT (text));
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
+    g_assert (IBUS_IS_TEXT (text));
 
     g_object_ref (text);
     g_array_append_val (table->labels, text);
@@ -347,7 +364,7 @@ IBusText *
 ibus_lookup_table_get_label (IBusLookupTable *table,
                              guint            index)
 {
-    g_return_val_if_fail (IBUS_IS_LOOKUP_TABLE (table), NULL);
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
 
     if (index >= table->labels->len)
         return NULL;
@@ -358,7 +375,7 @@ ibus_lookup_table_get_label (IBusLookupTable *table,
 void
 ibus_lookup_table_clear (IBusLookupTable *table)
 {
-    g_return_if_fail (IBUS_IS_LOOKUP_TABLE (table));
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
 
     gint index;
 
@@ -432,6 +449,23 @@ ibus_lookup_table_get_page_size (IBusLookupTable *table)
     return table->page_size;
 }
 
+void
+ibus_lookup_table_set_round (IBusLookupTable *table,
+                             gboolean         round)
+{
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
+
+    table->round = round ? TRUE: FALSE;
+}
+
+gboolean
+ibus_lookup_table_is_round (IBusLookupTable *table)
+{
+    g_assert (IBUS_IS_LOOKUP_TABLE (table));
+
+    return table->round;
+}
+
 gboolean
 ibus_lookup_table_page_up (IBusLookupTable *table)
 {
