@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -64,6 +65,7 @@ typedef struct _IBusLookupTableClass IBusLookupTableClass;
  * @cursor_pos: position index of cursor.
  * @cursor_visible: whether the cursor is visible.
  * @round: TRUE for lookup table wrap around.
+ * @orientation: orientation of the table.
  * @candidates: Candidate words/phrases.
  * @labels: Candidate labels which identify individual candidates in the same page. Default is 1, 2, 3, 4 ...
  *
@@ -81,6 +83,7 @@ struct _IBusLookupTable {
     guint cursor_pos;
     gboolean cursor_visible;
     gboolean round;
+    gint orientation;
 
     GArray *candidates;
     GArray *labels;
@@ -119,6 +122,16 @@ void                 ibus_lookup_table_append_candidate
                                                 (IBusLookupTable    *table,
                                                  IBusText           *text);
 
+/**
+ * ibus_lookup_table_get_number_of_candidates:
+ * @table: An IBusLookupTable.
+ * @returns: The number of candidates in the table
+ *
+ * Return the number of candidate in the table.
+ */
+guint               ibus_lookup_table_get_number_of_candidates
+                                                (IBusLookupTable    *table);
+
 /**
  * ibus_lookup_table_get_candidate:
  * @table: An IBusLookupTable.
@@ -229,6 +242,27 @@ void                 ibus_lookup_table_set_page_size
 guint                ibus_lookup_table_get_page_size
                                                 (IBusLookupTable    *table);
 
+/**
+ * ibus_lookup_table_set_round:
+ * @table: An IBusLookupTable.
+ * @round: Whether to make @table round.
+ *
+ * Set whether to make the IBusLookupTable round or not.
+ */
+void                 ibus_lookup_table_set_round
+                                                (IBusLookupTable    *table,
+                                                 gboolean            round);
+/**
+ * ibus_lookup_table_is_round:
+ * @table: An IBusLookupTable.
+ * @returns: Whether the @table is round.
+ *
+ * Returns whether the IBusLookupTable is round.
+ */
+gboolean             ibus_lookup_table_is_round (IBusLookupTable    *table);
+
+
+
 /**
  * ibus_lookup_table_clear:
  * @table: An IBusLookupTable.
