@@ -96,7 +96,15 @@ extern void ibus_marshal_VOID__UINT_UINT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:UINT,UINT,UINT (./ibusmarshalers.list:14) */
+/* VOID:INT,UINT (./ibusmarshalers.list:14) */
+extern void ibus_marshal_VOID__INT_UINT (GClosure     *closure,
+                                         GValue       *return_value,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint,
+                                         gpointer      marshal_data);
+
+/* VOID:UINT,UINT,UINT (./ibusmarshalers.list:15) */
 extern void ibus_marshal_VOID__UINT_UINT_UINT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -104,7 +112,7 @@ extern void ibus_marshal_VOID__UINT_UINT_UINT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT,BOOL (./ibusmarshalers.list:15) */
+/* VOID:OBJECT,UINT,BOOL (./ibusmarshalers.list:16) */
 extern void ibus_marshal_VOID__OBJECT_UINT_BOOLEAN (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -113,7 +121,7 @@ extern void ibus_marshal_VOID__OBJECT_UINT_BOOLEAN (GClosure     *closure,
                                                     gpointer      marshal_data);
 #define ibus_marshal_VOID__OBJECT_UINT_BOOL	ibus_marshal_VOID__OBJECT_UINT_BOOLEAN
 
-/* VOID:OBJECT,BOOL (./ibusmarshalers.list:16) */
+/* VOID:OBJECT,BOOL (./ibusmarshalers.list:17) */
 extern void ibus_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -122,7 +130,7 @@ extern void ibus_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                gpointer      marshal_data);
 #define ibus_marshal_VOID__OBJECT_BOOL	ibus_marshal_VOID__OBJECT_BOOLEAN
 
-/* VOID:BOXED,BOOL (./ibusmarshalers.list:17) */
+/* VOID:BOXED,BOOL (./ibusmarshalers.list:18) */
 extern void ibus_marshal_VOID__BOXED_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -131,10 +139,10 @@ extern void ibus_marshal_VOID__BOXED_BOOLEAN (GClosure     *closure,
                                               gpointer      marshal_data);
 #define ibus_marshal_VOID__BOXED_BOOL	ibus_marshal_VOID__BOXED_BOOLEAN
 
-/* VOID:BOXED (./ibusmarshalers.list:18) */
+/* VOID:BOXED (./ibusmarshalers.list:19) */
 #define ibus_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED
 
-/* VOID:STRING,STRING,BOXED (./ibusmarshalers.list:19) */
+/* VOID:STRING,STRING,BOXED (./ibusmarshalers.list:20) */
 extern void ibus_marshal_VOID__STRING_STRING_BOXED (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -142,7 +150,7 @@ extern void ibus_marshal_VOID__STRING_STRING_BOXED (GClosure     *closure,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING (./ibusmarshalers.list:20) */
+/* VOID:STRING,STRING,STRING (./ibusmarshalers.list:21) */
 extern void ibus_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
@@ -150,10 +158,10 @@ extern void ibus_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                      gpointer      invocation_hint,
                                                      gpointer      marshal_data);
 
-/* VOID:UINT (./ibusmarshalers.list:21) */
+/* VOID:UINT (./ibusmarshalers.list:22) */
 #define ibus_marshal_VOID__UINT	g_cclosure_marshal_VOID__UINT
 
-/* VOID:UINT,POINTER (./ibusmarshalers.list:22) */
+/* VOID:UINT,POINTER (./ibusmarshalers.list:23) */
 #define ibus_marshal_VOID__UINT_POINTER	g_cclosure_marshal_VOID__UINT_POINTER
 
 G_END_DECLS
