@@ -1,6 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -96,6 +97,8 @@ typedef enum
  * @IBUS_CAP_LOOKUP_TABLE: UI is capable to show the lookup table.
  * @IBUS_CAP_FOCUS: UI is capable to get focus.
  * @IBUS_CAP_PROPERTY: UI is capable to have property.
+ * @IBUS_CAP_SURROUNDING_TEXT: Client can provide surround text,
+ *  or IME can handle surround text.
  *
  * Capability flags of UI.
  */
@@ -105,8 +108,23 @@ typedef enum {
     IBUS_CAP_LOOKUP_TABLE       = 1 << 2,
     IBUS_CAP_FOCUS              = 1 << 3,
     IBUS_CAP_PROPERTY           = 1 << 4,
+    IBUS_CAP_SURROUNDING_TEXT   = 1 << 5,
 } IBusCapabilite;
 
+/**
+ * IBusOrientation:
+ * @IBUS_ORIENTATION_HORIZONTAL: Horizontal orientation.
+ * @IBUS_ORIENTATION_VERTICAL: Vertival orientation.
+ * @IBUS_ORIENTATION_SYSTEM: Use ibus global orientation setup.
+ *
+ * Orientation of UI.
+ */
+typedef enum {
+    IBUS_ORIENTATION_HORIZONTAL = 0,
+    IBUS_ORIENTATION_VERTICAL   = 1,
+    IBUS_ORIENTATION_SYSTEM     = 2,
+} IBusOrientation;
+
 /**
  * IBusRectangle:
  * @x: x coordinate.
