@@ -2,7 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright(c) 2007-2008 Huang Peng <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2009 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -27,6 +28,10 @@ import pango
 import ibus
 from ibus._gtk import PangoAttrList
 
+from gettext import dgettext
+_  = lambda a : dgettext("ibus", a)
+N_ = lambda a : a
+
 class EventBox(gtk.EventBox):
     __gtype_name__ = "IBusEventBox"
     def __init__(self):
@@ -190,7 +195,6 @@ class CandidatePanel(gtk.VBox):
     def __init__(self):
         super(CandidatePanel, self).__init__()
         self.set_name("IBusCandidate")
-        self.__tooltips = gtk.Tooltips()
 
         self.__toplevel = gtk.Window(gtk.WINDOW_POPUP)
         self.__viewport = gtk.Viewport()
@@ -240,7 +244,6 @@ class CandidatePanel(gtk.VBox):
         self.__aux_label.set_attributes(self.__aux_attrs)
         self.__aux_label.set_alignment(0.0, 0.5)
         self.__aux_label.set_padding(8, 0)
-        self.__tooltips.set_tip(self.__aux_label, "Aux string")
         self.__aux_label.set_no_show_all(True)
         if self.__aux_string_visible:
             self.__aux_label.show()
@@ -259,12 +262,12 @@ class CandidatePanel(gtk.VBox):
         self.__prev_button = gtk.Button()
         self.__prev_button.connect("clicked", lambda x: self.emit("page-up"))
         self.__prev_button.set_relief(gtk.RELIEF_NONE)
-        self.__tooltips.set_tip(self.__prev_button, "Previous page")
+        self.__prev_button.set_tooltip_text(_("Previous page"))
 
         self.__next_button = gtk.Button()
         self.__next_button.connect("clicked", lambda x: self.emit("page-down"))
         self.__next_button.set_relief(gtk.RELIEF_NONE)
-        self.__tooltips.set_tip(self.__next_button, "Next page")
+        self.__next_button.set_tooltip_text(_("Next page"))
 
         self.__pack_all_widgets()
 
