@@ -2,7 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright(c) 2007-2008 Huang Peng <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2009 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -53,7 +54,10 @@ class LanguageBar(gtk.Toolbar):
             gobject.SIGNAL_RUN_LAST,
             gobject.TYPE_PYOBJECT,
             ()),
-
+        "position-changed" : (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_INT, gobject.TYPE_INT)),
         }
 
     def __init__ (self):
@@ -72,15 +76,10 @@ class LanguageBar(gtk.Toolbar):
         self.__toplevel = gtk.Window(gtk.WINDOW_POPUP)
         self.__toplevel.connect("size-allocate", self.__toplevel_size_allocate_cb)
         self.__toplevel.add(self)
+        self.__screen = gdk.screen_get_default()
+        self.__screen.connect("size-changed", self.__screen_size_changed_cb)
 
-        root = gdk.get_default_root_window()
-        try:
-            self.__work_area = root.property_get("_NET_WORKAREA")[2]
-        except:
-            w, h = root.get_size()
-            self.__work_area = 0, 0, w, h
-        self.__position = self.__work_area[0] + self.__work_area[2] - 20, self.__work_area[1] + self.__work_area[3] - 20
-        self.__toplevel.move(*self.__position)
+        self.set_position(-1, -1)
 
     def __create_ui(self):
         # create move handle
@@ -103,6 +102,9 @@ class LanguageBar(gtk.Toolbar):
         self.__about_button.connect("clicked", self.__about_button_clicked_cb)
         self.insert(self.__about_button, -1)
 
+    def __screen_size_changed_cb(self, screen):
+        self.set_position(*self.__position)
+
     def __im_menu_toggled_cb(self, widget):
         if self.__im_menu.get_active():
             menu = self.emit("get-im-menu")
@@ -124,10 +126,12 @@ class LanguageBar(gtk.Toolbar):
         x, y = self.__toplevel.get_position()
         w, h = self.__toplevel.get_size()
         self.__position = x + w, y + h
+        self.emit("position-changed", *self.__position)
 
     def __toplevel_size_allocate_cb(self, toplevel, allocation):
         x, y = self.__position
-        if x - self.__work_area[0] >= self.__work_area[2] - 80:
+        w, h = self.__screen.get_width(), self.__screen.get_height()
+        if x >= w - 80 or True:
             self.__toplevel.move(x - allocation.width, y - allocation.height)
 
     def __remove_properties(self):
@@ -194,6 +198,20 @@ class LanguageBar(gtk.Toolbar):
         else:
             self.focus_out()
 
+    def set_position(self, x, y):
+        w, h = self.__screen.get_width(), self.__screen.get_height()
+        if x < 0 or y < 0:
+            x = w - 20
+            y = h - 40
+        if x > w:
+            x = w - 20
+        if y > h:
+            y = h - 40
+
+        self.__position = x, y
+        w, h = self.__toplevel.get_size()
+        self.__toplevel.move(self.__position[0] - w, self.__position[1] - h)
+
     def get_show(self):
         return self.__show
 
