@@ -2,7 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright(c) 2007-2008 Huang Peng <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2009 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,7 +25,7 @@ import sys
 import getopt
 import ibus
 import gtk
-import locale
+import gettext
 import panel
 import notifications
 
@@ -76,5 +77,7 @@ def main():
     launch_panel()
 
 if __name__ == "__main__":
-    locale.bind_textdomain_codeset("ibus", "UTF-8")
+    localedir = os.getenv("IBUS_LOCALEDIR")
+    gettext.bindtextdomain("ibus", localedir)
+    gettext.bind_textdomain_codeset("ibus", "UTF-8")
     main()
